@@ -6,12 +6,14 @@ use Carp;
 use Template::Tiny;
 use Safe::Isa;
 
-use Moo;
+use Text::Tabs qw[ expand ];
 use Types::Standard -all;
 use Type::Params qw[ validate ];
 
+use Moo;
 
-our $VERSION = '0.07'; # TRIAL
+
+our $VERSION = '0.09'; # TRIAL
 
 BEGIN {
     if ( $^O =~ /Win32/i ) {
@@ -23,12 +25,51 @@ use Term::ANSIColor ();
 use namespace::clean;
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 has pipe => (
     is       => 'rw',
     isa      => InstanceOf ['IPC::PrettyPipe'],
     required => 1,
 );
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 has colors => (
     is      => 'rw',
     isa     => HashRef,
@@ -57,38 +98,72 @@ has colors => (
 );
 
 
-has template => (
 
-    is => 'rw',
 
-    default => sub {
-        return <<"EOT"
-[% IF pipe.streams.empty %][% ELSE %](\t\\
-[% END -%]
-[%- FOREACH cmd IN pipe.cmds.elements %]
-[%- IF cmd.first %]  [% ELSE %]\t\\
-| [% END %][% color.cmd.cmd %][% cmd.quoted_cmd %][% color.reset %]
-[%- FOREACH arg IN cmd.args.elements %]\t\\
-    [% color.cmd.arg.pfx %][% arg.pfx %]
-[%- color.cmd.arg.name %][% arg.quoted_name %]
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+has cmd_template => (
+    is      => 'rw',
+    default => <<"EOT",
+[%- indent %][% color.cmd.cmd %][% cmd.quoted_cmd %][% color.reset %]
+[%- FOREACH arg IN cmd.args.elements %]\\
+[% indent %][% indent %][% color.cmd.arg.pfx %][% arg.pfx %][% color.cmd.arg.name %][% arg.quoted_name %]
 [%- IF arg.has_value %][%- IF arg.sep %][% color.cmd.arg.sep %][% arg.sep %][% ELSE %] [% END %]
 [%- color.cmd.arg.value %][% arg.quoted_value %][% END %][% color.reset %]
 [%- END %]
-[%- FOREACH stream IN cmd.streams.elements %]\t\\
-[% color.cmd.stream.spec %][% stream.spec %]
-[%- IF stream.has_file %] [% END %][% color.cmd.stream.file %][% stream.quoted_file %][% color.reset %]
-[%- END %]
-[%- END %]
-[%- IF pipe.streams.empty %][% ELSE %]\t\\
-)[% FOREACH stream IN pipe.streams.elements %]\t\\
-[% color.pipe.stream.spec %][% stream.spec -%]
-[%- IF stream.has_file %] [% END %][% color.pipe.stream.file %][% stream.quoted_file %][% color.reset %]
-[%- END %]
-[%- END %]
+[%- FOREACH stream IN cmd.streams.elements %]\\
+[% indent %][% indent %][% color.cmd.stream.spec %][% stream.spec -%]
+[% IF stream.has_file %] [% color.cmd.stream.file %][% stream.quoted_file %][% color.reset %][% END %]
+[%- END -%]
 EOT
-    },
+);
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+has pipe_template => (
+
+    is => 'rw',
+
+    default => <<"EOT",
+[% indent %][% IF pipe.streams.empty %][% ELSE %](\\
+[% END %][% cmds %]
+[%- IF pipe.streams.empty %][% ELSE %]\\
+[% indent %])[% FOREACH stream IN pipe.streams.elements -%]
+[% IF stream.first %]\t[% ELSE %]\\
+\t[% indent %][% END -%]
+[% color.pipe.stream.spec %][% stream.spec -%]
+[% IF stream.has_file %] [% color.pipe.stream.file %][% stream.quoted_file %][% color.reset %][% END %]
+[%- END -%]
+[%- END -%]
+EOT
 );
 
 sub _colorize {
@@ -114,43 +189,100 @@ sub _colorize {
 
 }
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 sub render {
 
     my $self = shift;
 
-    my ( $args ) =
-      validate( \@_,
-                slurpy Dict[
-                            colorize => Optional[ Bool ],
-                           ]
-              );
-
-
+    my ( $args ) = validate(
+        \@_,
+        slurpy Dict [
+            colorize => Optional [Bool],
+        ] );
 
     $args->{colorize} //= 1;    ## no critic (ProhibitAccessOfPrivateData)
 
-    my $output;
-
     my %color;
     _colorize( $self->colors, \%color );
 
     $color{reset} = Term::ANSIColor::color( 'reset' )
       if keys %color;
 
+    local $Text::Tabs::tabstop = 2;
 
-    Template::Tiny->new->process(
-        \$self->template,
-        {
-         ## no critic (ProhibitAccessOfPrivateData)
-            pipe => $self->pipe,
-            $args->{colorize} ? ( color => \%color ) : (),
-        },
-        \$output,
-    );
+    # generate non-colorized version so can get length of records to
+    # pad out any continuation lines
+    my @output;
+    $self->_render_pipe( $self->pipe, { indent => '' },  \@output);
 
-    return $output;
+    my @records = map { expand( $_ ) } map { split( /\n/, $_ ) } @output;
+    my @lengths = map { length } @records;
+    my $max = List::Util::max( @lengths ) + 4;
+
+    if ( $args->{colorize} ) {
+        @output = ();
+        $self->_render_pipe( $self->pipe, { indent => '', color => \%color },  \@output);
+        @records = map { expand( $_ ) } map { split( /\n/, $_ ) } @output;
+    }
+
+    foreach ( @records ) {
+        my $pad =  ' ' x ($max - shift @lengths);
+        s/\\$/$pad \\/;
+    }
+
+    return join ("\n", @records ) . "\n";
 }
 
+sub _render_pipe {
+
+    my $self = shift;
+
+    my ( $pipe, $process_args, $output ) = @_;
+
+    my %process_args = %{$process_args};
+
+    my @output;
+
+    for my $cmd ( @{ $pipe->cmds->elements } ) {
+        if ( $cmd->isa( 'IPC::PrettyPipe' ) ) {
+            local $process_args{indent} = $process_args{indent} . "\t";
+            $self->_render_pipe( $cmd, \%process_args, \@output );
+        }
+        else {
+            local $process_args{indent} = $process_args{indent} . "\t";
+            push @output, '';
+            local $process_args{cmd} = $cmd;
+            Template::Tiny->new->process( \$self->cmd_template, \%process_args,
+                \( $output[-1] ) );
+        }
+    }
+
+    $process_args{cmds} = join( "\\\n|", @output );
+    $process_args{pipe} = $pipe;
+    push @{$output}, '';
+    Template::Tiny->new->process( \$self->pipe_template, \%process_args,
+        \( $output->[-1] ) );
+    return;
+}
+
+
 with 'IPC::PrettyPipe::Renderer';
 
 1;
@@ -177,7 +309,7 @@ IPC::PrettyPipe::Render::Template::Tiny - rendering backend using B<Template::Ti
 
 =head1 VERSION
 
-version 0.07
+version 0.09
 
 =head1 SYNOPSIS
 
@@ -193,38 +325,68 @@ version 0.07
 =head1 DESCRIPTION
 
 B<IPC::PrettyPipe::Render::Template::Tiny> implements the
-B<L<IPC::PrettyPipe::Renderer>> role, providing a rendering backend for
-B<L<IPC::PrettyPipe>> using the B<L<Template::Tiny>> module.
+L<IPC::PrettyPipe::Renderer> role, providing a rendering backend for
+L<IPC::PrettyPipe> using the L<Template::Tiny> module.
 
-=head1 METHODS
+=head1 ATTRIBUTES
 
-=over
+=head2 pipe
 
-=item new
+The L<IPC::PrettyPipe> object to render.
 
-  $renderer = IPC::PrettyPipe::Render::Template::Tiny->new( %attr );
+=head2 pipe
 
-Construct a new renderer.  Typically this is done automatically by B<L<IPC::PrettyPipe>>.
-The following attributes are available:
+  $pipe = $renderer->pipe;
+  $renderer->pipe( $pipe );
 
-=over
+Retrieve or set the L<IPC::PrettyPipe> object to render.
 
-=item pipe
-
-The B<L<IPC::PrettyPipe>> object to render.
-
-=item colors
+=head2 colors
 
 A color specification; see L</Rendered Colors>.
 
-=item template
+=head2 cmd_template
 
-A B<L<Template::Tiny>> template to generate the output.  See L</Rendering
-Template>.
+A L<Template::Tiny> template to generate output for commands.  See L</Rendering
+Templates>.
 
-=back
+=head2 pipe_template
 
-=item render
+A L<Template::Tiny> template to generate output for pipes.  See L</Rendering
+Templates>.
+
+=head1 METHODS
+
+=head2 new
+
+  $renderer = IPC::PrettyPipe::Render::Template::Tiny->new( %attr );
+
+Construct a new renderer.  Typically this is done automatically by L<IPC::PrettyPipe>.
+
+=head2 colors
+
+  $colors = $renderer->colors;
+  $renderer->colors( $colors );
+
+Retrieve or set the colors to be output; see L</Rendered Colors>.
+
+=head2 cmd_template
+
+  $cmd_template = $renderer->cmd_template;
+  $renderer->cmd_template( $cmd_template );
+
+Retrieve or set the L<Template::Tiny> template used to generate output
+for commands.  See L</Rendering Templates>.
+
+=head2 pipe_template
+
+  $pipe_template = $renderer->pipe_template;
+  $renderer->pipe_template( $pipe_template );
+
+Retrieve or set the L<Template::Tiny> template used to generate output
+for pipes.  See L</Rendering Templates>.
+
+=head2 render
 
   $renderer->render( %options );
 
@@ -236,55 +398,27 @@ The following options are available:
 
 colorize
 
-If true (the default) the output is colorized using B<L<Term::AnsiColor>>.
-
-=back
-
-=item pipe
-
-  $pipe = $renderer->pipe;
-  $renderer->pipe( $pipe );
-
-Retrieve or set the B<L<IPC::PrettyPipe>> object to render.
-
-=item colors
-
-  $colors = $renderer->colors;
-  $renderer->colors( $colors );
-
-Retrieve or set the colors to be output; see L</Rendered Colors>.
-
-=item template
-
-  $template = $renderer->template;
-  $renderer->template( $template );
-
-Retrieve or set the B<L<Template::Tiny>> template used to generate the
-output.  See L</Rendering Template>.
+If true (the default) the output is colorized using L<Term::AnsiColor>.
 
 =back
 
 =head1 CONFIGURATION
 
-=head2 Rendering Template
+=head2 Rendering Templates
 
-The C<template> attribute and method may be used to change the
-template used to render the pipeline.  The template is passed the
-following parameters:
+Because pipes may be nested and L<Template::Tiny> cannot handle
+recursive logic, there are two templates, C<cmd_template> for commands
+and C<pipe_template> for pipes.
 
-=over
-
-=item C<pipe>
-
-C<pipe> is the B<L<IPC::PrettyPipe>> object. B<L<Template::Tiny>> doesn't
-support loop constructs, so the B<L<IPC::PrettyPipe>> B<L<streams|IPC::PrettyPipe/streams>> and
-B<L<cmds|IPC::PrettyPipe/cmds>> methods return B<L<IPC::PrettyPipe::Queue>> objects, which
-provide methods for determining if the lists are empty.
+L<Template::Tiny> also doesn't support loop constructs, so the
+L<IPC::PrettyPipe> L<streams|IPC::PrettyPipe/streams> and
+L<cmds|IPC::PrettyPipe/cmds> methods return L<IPC::PrettyPipe::Queue>
+objects, which provide methods for determining if the lists are empty.
 
   [% IF pipe.streams.empty %][% ELSE %](\t\\
   [% END -%]
 
-Note that B<L<Template::Tiny>> resolves object methods with the same syntax as it
+Note that L<Template::Tiny> resolves object methods with the same syntax as it
 resolves hash entries.
 
 Iteration looks like this:
@@ -292,12 +426,38 @@ Iteration looks like this:
   [%- FOREACH cmd IN pipe.cmds.elements %]
   [% END %]
 
-An B<L<IPC::PrettyPipe::Queue::Element>> has additional methods which
+An L<IPC::PrettyPipe::Queue::Element> has additional methods which
 indicates whether it is the first or last in its queue.
 
   [%- IF cmd.first %]  [% ELSE %]\t\\
   | [% END %]
 
+The templates are passed the following parameters:
+
+=over
+
+=item C<indent>
+
+Indentation is performed via tab stops.  C<indent> is augmented with
+additional tab characters (C<\t>) for nested pipes.
+
+=item C<pipe>
+
+This is passed only to the C<pipe_template>.
+
+C<pipe> is the L<IPC::PrettyPipe> object.
+
+=item C<cmd>
+
+This is passed only to the C<cmd_template>
+
+C<cmd> is the L<IPC::PrettyPipe::Cmd> object.
+
+=item C<cmds>
+
+This is passed only to the C<pipe_template>.
+It contains the rendered commands in the pipe.
+
 =item C<color>
 
 This is the hashref specified by the C<colors> attribute or method.
@@ -309,9 +469,9 @@ and colorize a command:
 
 =back
 
-The default template (encoded into the source file for
-B<IPC::PrettyPipe::Render::Template::Tiny>) provides a good example
-of how to construct one.
+Use the default templates (encoded into the source file for
+B<IPC::PrettyPipe::Render::Template::Tiny>) as a basis for
+exploration.
 
 =head2 Rendered Colors
 
